
// bibhtml: Generates HTML page (with images) from BibTex files
// See: https://www.telecom-paris.fr/~elc/software/
//
// Copyright 2020 Eric Lecolinet (eric.lecolinet@gmail.com)
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef __Bib__
#define __Bib__

#include <vector>
#include <map>
#include "bibdefs.hpp"
#include "bibentry.hpp"

class BibStyle;
class BibExtension;


/// Specify whether entry images are not shown or shown on the left or rigt side of the entry.
enum class ShowImages{None, Left, Right};

/// Specify whether entry links (pdf, etc.) are not shown or shown as text or as icons.
enum class ShowLinks{None, Text, Icon};


/** @brief Main class of the program.
 */
class Bib {
public:
  typedef std::map<std::string, BibAuthors> AuthorMap;
  typedef std::map<std::string, BibGroup*> GroupMap;
  typedef std::map<std::string, BibEntry*> EntryMap;
  
  static Bib instance;
  static std::list<BibExtension*> extensions;

  ~Bib();
  void init(int argc, char *argv[]);
  
  bool interMode() const {return interMode_;}
  bool verboseMode() const {return verboseMode_;}
  bool breakLines() const {return breakLines_;}
  bool showNumbers() const {return showNumbers_;}
  ShowImages showImages() const {return showImages_;}
  ShowLinks showLinks() const {return showLinks_;}
  BibFields::ID showHeaders() const {return showHeaders_;}
  const std::string& imageDir() const {return imageDir_;}
  const std::string& htmlBegin() const {return htmlBegin_;}
  const std::string& htmlEnd() const {return htmlEnd_;}
  const BibEntries& medias() const {return medias_;}
  
  // - - Entries - -
  
  /** returns an entry after creating it if needed.
   *  If an entry with the same bibkey already exists, it is returned without creating
   *  a new entry.
   *  Entries can have any type (the type is automatically added to BibTypes if needed).
   *  _isAuxiliary_ should be true if this entry comes from an auxiliary file (such
   *  entries are ignored when printing  HTML files).
   */
  BibEntry* addEntry(const std::string& bibkey, const std::string& typeName, bool isAuxiliary);
  
  /// post-treatment to complete the entry.
  void completeEntry(BibEntry&);

  /// Returns the entry with this bibkey.
  const BibEntry* getEntry(const std::string& bibkey) const;
  
  /** @brief Selects entries.
   *  The query syntax is: field1 & field2=value2 & field3!=value3 & field4=value4a|!value4b ...
   *  All conditions betwen "&" symbols must be met. These conditions mean:
   *  - field1 : field1 is unempty
   *  - field2=value2 : field2 contains value2
   *  - field3!=value3 : field3 does not contains value3
   *  - field4=value4a|!value4b : field4 contains value4a OR does not contain value4b
   * Case does not matter. Field names can be abridged.
   */
  void select(const std::string& query_pattern);
  
  /** @brief Sorts entries.
   *  Example: "-year & type"
   *  - _sortPattern_ can contain one or several BibTex fields separated by the 
   *    "&" symbol, meaning (in this example) that the papers that are currently
   *    selected will be sorted by year then by type.
   *  - The "-" symbol before "year" reverses order (greater dates first).
   * Case does not matter. Field names can be abridged.
   */
  void sort(const std::string& sort_pattern);
  
  /// Clears the entry database.
  void clear();
  
  /** @brief Reads a BibTex file.
   *  Reads all entries in this file and adds them to the entry database.
   *  Duplicate entries are merged taking into account their key. If an entry
   *  field if defined multiple times, the last version is kept.
   *  _isAuxiliary_ must be true if this file is an auxiliary file (its entries
   *  may be merged with existing entries, otherwise they will be ignored when
   *  printing  HTML files).
   */
  void readBib(const std::string& fileName, bool isAuxiliary);
  
  /// Changes the name of the generated HTML file.
  void setHtmlFile(const std::string& fileName);

  /// Changes the name of the generated BibTex file.
  void setBibFile(const std::string& fileName);
  
  /** @brief Writes the selected entries in the generated HTML file.
   *  Displays the HTML file using the default Web browser if _display_ is true.
   *  @note Writes only entries that are currently selected (not the whole entry database)
   */
  void writeHtml(bool display);
  
  /** @brief Write the selected entries in the generated BibText file.
   *  Displays the BibText file using the default Web browser if _display_ is true.
   *  @note Writes only entries that are currently selected (not the whole entry database)
   */
  void writeBib(bool display);
  
  // - - Authors - -
  
  BibAuthor* addAuthor(const std::string& lastname, const std::string& firstname);
  
  BibAuthor* getAuthor(const std::string& lastname, const std::string& firstname) const;
  
  const Bib::AuthorMap& authors() const {return authorMap_;}

  // - - Formatting - -

  /// Reads the HTML template that serves to produce the HTML output file.
  void readTemplate(const std::string& fileName);

  /// Changes formatting options, including the printing Style.
  void setFormat(const std::string& options);
  
  /// Changes the Style (see BibStyle).
  void setStyle(const std::string& styleKey);

  // - - Misc. - -

  /// Checks if links corresponding to this field are valid, removes them otherwise.
  void checkLinks(const std::string& fieldName);
  
  /// Shows statitistics.
  void showStats(std::string filename);

  /// Executes a command.
  void doCommand(const std::string& cmd, const std::string& args, bool& interFound, bool& showFound);
  
  /// Prints a message on the console.
  static void echo(const std::string& msg);

  /// Prints an error on the console.
  static void error(const char* command, const std::string& msg);
  
  /// Prints help on the console.
  static bool help();
  
private:
  friend class BibStats;
  bool
  templateSearched_{false}, hasMedias_{false},
  interMode_{false}, verboseMode_{false},
  breakLines_{true}, showNumbers_{false};
  ShowImages    showImages_{ShowImages::Left};
  ShowLinks     showLinks_{ShowLinks::Text};
  BibFields::ID showHeaders_{BibFields::ID::year};
  std::string   imageDir_;
  std::string   outHtml_{BibSettings::OutHtml};
  std::string   outBib_{BibSettings::OutBib};
  std::string   htmlBegin_, htmlEnd_;
  AuthorMap     authorMap_;
  GroupMap      groupMap_;
  EntryMap      entryMap_, styleMap_;
  BibEntries    *selectEntries_{nullptr}, allEntries_, medias_;
  BibStyle      *style_{nullptr};
  
  class Commands {
#define BIBCOMMANDS \
read, aux, images, templ, out, outhtml, outbib, format, all, select, sort,\
clear, show, showhtml, showbib, check, stats, help, i, inter, verbose, quit
  public:
    enum ID {BIBCOMMANDS};   // command IDs
    ccuty::stringset names;  // command names
    Commands() : names(STRINGIFY(BIBCOMMANDS)) {}
  };
  Commands commands_;
#undef BIBCOMMANDS
};

#endif
