
// bibhtml: Generates HTML page (with images) from BibTex files
// See: https://www.telecom-paris.fr/~elc/software/
//
// Copyright 2020 Eric Lecolinet (eric.lecolinet@gmail.com)
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#include <iostream>
#include <fstream>
#include "bib.hpp"
#include "bibstats.hpp"
using namespace std;


// stats by entry type
struct TypeStats {
  TypeStats() : total(0), toappear(0), unknown_rank(0), ranks(8) {}
  int total, toappear, unknown_rank;
  std::vector<int> ranks;
};


// stats per user or per Authgroup
class AuthStats {
public:
  AuthStats() : total(0), toappear(0) {}
  int total, toappear;
  std::map<std::string, int> years_total, years_toappear;
};


BibStats::BibStats(const std::string& filename,
                   BibEntries& entries, Bib::GroupMap& groups, bool with_ranks)
{

  std::ofstream f(filename);
  if (!f) {
    Bib::error("stats","can't open '"+filename+"' aborting");
    return;
  }
  
  f << "\nStats:\n" << endl;
  
  vector<TypeStats> tstats_vect(BibTypes::count());
  int entry_count = 0;
  int entry_toappear = 0;
  int min_year = 3000, max_year = 0;
  
  if (with_ranks) f << "Type;Count;Unknown;0;1;2;3;4;5;6;7" << endl;
  else f << "Type;Count" << endl;
  
  for (auto& entry : entries) {
    
    // entry.index_ == 0 : do not count per author duplicates!
    if (entry.index_ == 0 ) {
      int typeID = entry.getTypeID();
      if (typeID >= BibTypes::count()) {
        Bib::error("stats","wrong entry type = " + to_string(typeID) + " ignoring entry");
        continue;
      }
      
      // stats by entry type
      int toappear = !entry.field(BibFields::pubstate).empty();
      TypeStats& tstats = tstats_vect[typeID];
      tstats.total++;
      tstats.toappear += toappear;
      
      // total count
      entry_count++;
      entry_toappear += toappear;
      
      // ranks
      if (with_ranks) {
        const string& rank_str = entry.field(BibFields::rank);
        int rank = -1;
        if (!rank_str.empty() && isdigit(rank_str[0])) {
          try {rank = std::stoi(rank_str);}
          catch (...) {Bib::error("stats","wrong rank = " + rank_str);}
        }
        if (rank >= 0 && rank < tstats.ranks.size()) (tstats.ranks[rank])++;
        else tstats.unknown_rank++;
      }
      
      std::string year = entry.field(BibFields::year);
      // find min and max year
      try {
        int yval = std::stoi(year);
        min_year = std::min(min_year, yval);
        max_year = std::max(max_year, yval);
      }
      catch (...) {Bib::error("stats","wrong year = " + year);}
      
      BibGroup* authgroup = nullptr;

      // stats per author
      for (auto& a : entry.authors()) {
        if (!a->stats_) a->stats_ = new AuthStats();
        AuthStats& astats = *a->stats_;
        astats.total++;
        astats.toappear += toappear;
        astats.years_total[year]++;
        astats.years_toappear[year] += toappear;
        
        if (!authgroup) authgroup = a->authgroup_;  //  !! we suppose all authors are in the same group !!
      }

      // stats per group
      if (authgroup) {
        if (!authgroup->stats_) authgroup->stats_ = new AuthStats();
        AuthStats& gstats = *authgroup->stats_;
        gstats.total++;
        gstats.toappear += toappear;
        gstats.years_total[year]++;
        gstats.years_toappear[year] += toappear;
      }
    }
  }
  
  // - - - show stats by entry type
  
  for (int type = 0; type < BibTypes::count(); ++type) {
    TypeStats& stats = tstats_vect[type];
    
    if (stats.total > 0) {    // ignore empty categories
      auto& type_name = BibTypes::typeName(BibTypes::ID(type));
      
      f << type_name << ";" << stats.total;
      if (with_ranks) {
        f << ";" << stats.unknown_rank;
        for (auto& r : stats.ranks) f << ";" << r;
      }
      f << endl;
    }
  }
  f << "Total Publications;" << entry_count
  << ";ToAppear;" << entry_toappear  << "\n" << endl;
  //f << min_year << " " << max_year << endl;
  
  // - - - show stats per user & per group
  
  int cumulated_count = 0;
  
  for (auto& it : groups) {
    BibGroup* gr = it.second;
    
    f << "\nGroup:" << gr->name() << "; ;";
    if (!gr->stats_) f << "0" << endl;
    else {
      f << gr->stats_->total << ";" <<  gr->stats_->toappear<< ";";
      
      for (int y = min_year; y <= max_year; ++y) {
        f << gr->stats_->years_total[to_string(y)] << ";"
        << gr->stats_->years_toappear[to_string(y)] << ";";
      }
      f << endl;
    }
    
    for (auto& a : gr->authors()) {
      f << a->lastname_ << ";" << a->firstname_<< ";";
      
      if (!a->stats_) f << "0" << endl;
      else {
        cumulated_count += a->stats_->total;
        f << a->stats_->total << ";" <<  a->stats_->toappear<< ";";
        
        for (int y = min_year; y <= max_year; ++y) {
          f << a->stats_->years_total[to_string(y)] << ";"
          << a->stats_->years_toappear[to_string(y)] << ";";
        }
        f << endl;
      }
    }
  }
  
  f << "\nTotal by Author;" << cumulated_count << "\n\n" << endl;
  
  // - - - show all authors
  
  for (auto& it : Bib::instance.authors()) {
    for (auto* a : it.second) {
      f << a->lastname_ << ";" << a->firstname_<< ";"
      << (a->authgroup_ ? a->authgroup_->name() : "") << endl;
    }
  }
}
