
// bibhtml: Generates HTML page (with images) from BibTex files
// See: https://www.telecom-paris.fr/~elc/software/
//
// Copyright 2020 Eric Lecolinet (eric.lecolinet@gmail.com)
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef __BibStyle__
#define __BibStyle__
#include <iostream>
#include <fstream>
#include <vector>
#include <map>
#include "bibentry.hpp"

class Bib;

/** @brief Printing style of an entry field.
 *  Defines the style of an entry field.
 */
class FieldStyle {
  friend class TypeStyle;
  enum {None, Separator, Text, Link, Button} type{None};
  BibFields::ID field{BibFields::none};
  bool breakline{false};
  std::string prefix, brprefix, name, img, suffix;
};


/** @brief Printing style of an entry type.
 *  Defines the style of an entry type. Contains the styles of the fields that
 *  are displayed for this entry type.
 */
class TypeStyle {
  friend class BibStyle;
  
  /// Parse this entry.
  void parse(Bib&, std::string pattern);
  
  /// Write this entry in a HMTL file.
  bool write(Bib&, std::ostream&, const BibEntry&, int number) const;
  
private:
  std::vector<FieldStyle*> fields;
};


/** @brief Printing style.
 *
 *  Defines a printing style. Contains the styles of the types that are defined
 *  for this printing style:
 *  - *must* always define a _media_ field. This field specifies the default style,
 *    which is used for entry types that do not have a specific definition.
 *  - *can* optionally define a style for each entry type. The field is the name
 *    of the entry type.
 *  - in both cases, the value is the corresponding style format.
 */
class BibStyle {
public:
  static BibStyle* createStyle(Bib&, const std::string& format);
  static BibStyle* createStyle(Bib&, const BibEntry& styleRef);

  /// Write all entries in a HMTL file.
  void write(Bib&, const std::string& filename, const BibEntries&) const;
  
private:
  TypeStyle defaultTypeStyle_;
  std::map<BibTypes::ID, TypeStyle> typeStyles_;
};

#endif
